@@ -88,6 +88,11 @@ int crypt_backend_init(struct crypt_device *ctx)
 	return 0;
 }
 
+void crypt_backend_destroy(void)
+{
+	crypto_backend_initialised = 0;
+}
+
 uint32_t crypt_backend_flags(void)
 {
 	return 0;
@@ -307,13 +312,24 @@ int crypt_pbkdf(const char *kdf, const char *hash,
 		const char *password, size_t password_length,
 		const char *salt, size_t salt_length,
 		char *key, size_t key_length,
-		unsigned int iterations)
+		uint32_t iterations, uint32_t memory, uint32_t parallel)
 {
-	struct hash_alg *ha = _get_alg(hash);
+	struct hash_alg *ha;
 
-	if (!ha || !kdf || strncmp(kdf, "pbkdf2", 6))
+	if (!kdf)
 		return -EINVAL;
 
-	return pkcs5_pbkdf2(hash, password, password_length, salt, salt_length,
-			    iterations, key_length, key, ha->block_length);
+	if (!strcmp(kdf, "pbkdf2")) {
+		ha = _get_alg(hash);
+		if (!ha)
+			return -EINVAL;
+
+		return pkcs5_pbkdf2(hash, password, password_length, salt, salt_length,
+				    iterations, key_length, key, ha->block_length);
+	} else if (!strncmp(kdf, "argon2", 6)) {
+		return argon2(kdf, password, password_length, salt, salt_length,
+			      key, key_length, iterations, memory, parallel);
+	}
+
+	return -EINVAL;
 }
